import numpy as np

import scipy.io as sio  # Matlab export

from keras.datasets import mnist
from keras.models import Sequential
from keras.utils import np_utils

# load MNIST data
(X_trn, t_trn), (X_tst, t_tst) = mnist.load_data()

# proportions
P = [0.005, 0.01, 0.1, 1.0]

batch_size = 100

# extract 0 to 5 data
X_trn05 = X_trn[t_trn < 6]
t_trn05 = t_trn[t_trn < 6]
X_tst05 = X_tst[t_tst < 6]
t_tst05 = t_tst[t_tst < 6]

# extract 0 to 6 data (dictionary using the proportion as a key)
X_trn69 = {}
t_trn69 = {}
n_69_samples = np.sum(t_trn[t_trn >= 6])  # the number of all 6-9 samples
for p in P:
    n_samples = batch_size * (int(n_69_samples * p) / batch_size)  # round to the batch size
    X_trn69[p] = X_trn[t_trn >= 6][:n_samples]
    t_trn69[p] = t_trn[t_trn >= 6][:n_samples] - 6
X_tst69 = X_tst[t_tst >= 6]
t_tst69 = t_tst[t_tst >= 6] - 6

# Matlab export
mdict = {'X_trn05': X_trn05, 't_trn05': t_trn05, 'X_tst05': X_tst05, 't_tst05': t_tst05, 'X_tst69': X_tst69,
         't_tst69': t_tst69}
for p in P:
    mdict['X_trn69_{}'.format(p)] = X_trn69[p]
    mdict['t_trn69_{}'.format(p)] = t_trn69[p]
sio.savemat('mnist.mat', mdict)


# convert to floats having range [0, 1]
def convert_to_float(array):
    return array.astype('float32') / 255


X_trn05 = convert_to_float(X_trn05)
X_tst05 = convert_to_float(X_tst05)
for p in P:
    X_trn69[p] = convert_to_float(X_trn69[p])
X_tst69 = convert_to_float(X_tst69)

# KERAS
# see https://keras.io/

convolution_layers = [
    # Convolution2D( ...),
    # ...
]

classification_layers = [
    # ...
]

model = Sequential(convolution_layers + classification_layers)

model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=['accuracy'])

# fit model using model.fit(..., batch_size=batch_size)
# use np_utils.to_categorical() to convert targets to the one hot encoding
# note that the evaluation of validation accuracy is affected when using dropout!
# you might use callback (https://keras.io/callbacks/) to store the accuracies

# use layer get_weights() and set_weights() methods to transfer the weights
# setting layer.trainable = False prevents parameter updates for the "layer"

# Note that Theano and TensorFlow uses different dimension ordering you might use the following
# from keras import backend as K
# if K.image_dim_ordering() == 'th':
#     input_shape = (1, img_rows, img_cols)
# else:
#     input_shape = (img_rows, img_cols, 1)